/******************************************************************************
CLASSE DIALCONFIGURACOES
CRIADO: 31-10-2008
ALTERADO: 04-01-2009
AUTOR: LUIS VASQUEZ - luis@comunidadeabiblia.net
WEB: www.comunidadeabiblia.net
--------------------------------aBblia Software-------------------------------
******************************************************************************/

#include "dialconfiguracoes.h"

#include "amessagebox.h"
#include "ferramentas.h"

DialConfiguracoes::DialConfiguracoes():DialBase(){
	setWindowTitle("Configuraes - " + (const QString&)NOME);
	setMinimumSize( QSize( 400, 455 ) );
	setMaximumSize( QSize( 400, 455 ) );

	QFont fonte = QFont("Bitstream", FONTE_MEDIA);
	boxTextos = new QGroupBox( this );
	boxTextos->setFont(fonte);
	boxTextos->setTitle("Textos");

	fonte = QFont("Bitstream", FONTE_PEQUENA);
	chkHabilitarTexto = new QCheckBox("Habilitar texto secundrio", boxTextos);
	chkHabilitarTexto->setFont(fonte);

	fonte = QFont("Bitstream", FONTE_MEDIA);
	boxVersoes = new QGroupBox( boxTextos );
	boxVersoes->setFont(fonte);
	boxVersoes->setTitle("Verses disponveis");

	cmbVersoesTexto = new QComboBox( boxVersoes );
	cmbVersoesTexto->clear();
	cmbVersoesTexto->addItem("J.F. Almeida Atualizada (1967)");
	cmbVersoesTexto->addItem("J.F. Almeida Corrigida e Fiel (1994)");
	cmbVersoesTexto->setFont(fonte);
	cmbVersoesTexto->setToolTip("Verses dos textos bblicos");
	
	boxTextoPrincipal = new QGroupBox( boxTextos );
	boxTextoPrincipal->setFont(fonte);
	boxTextoPrincipal->setTitle("Verso do Texto Primrio");

	ledTextoPrincipal = new QLineEdit( boxTextoPrincipal );
	ledTextoPrincipal->setReadOnly(TRUE);
	ledTextoPrincipal->setFont(fonte);
	ledTextoPrincipal->setText(config.getTextoPrincipal());
	ledTextoPrincipal->setToolTip("Verso definida como\ntexto primrio");

	btnTextoPrincipal = new QPushButton("Definir", boxTextoPrincipal );
	btnTextoPrincipal->setFont(fonte);
	btnTextoPrincipal->setToolTip("Definir verso selecionada\ncomo texto primrio");
	
	boxTextoSecundario = new QGroupBox( boxTextos );
	boxTextoSecundario->setFont(fonte);
	boxTextoSecundario->setTitle("Verso do Texto Secundrio");

	ledTextoSecundario = new QLineEdit( boxTextoSecundario );
	ledTextoSecundario->setReadOnly(TRUE);
	ledTextoSecundario->setFont(fonte);
	ledTextoSecundario->setText(config.getTextoSecundario());
	ledTextoSecundario->setToolTip("Verso definida como\ntexto secundrio");

	btnTextoSecundario = new QPushButton("Definir", boxTextoSecundario );
	btnTextoSecundario->setFont(fonte);
	btnTextoSecundario->setToolTip("Definir verso selecionada\ncomo texto secundrio");
	
	boxGeral = new QGroupBox( this );
	boxGeral->setFont(fonte);
	boxGeral->setTitle("Geral");
	
	btnConfigDefault = new QPushButton("Restaurar configuraes default", boxGeral );
	btnConfigDefault->setFont(fonte);
	btnConfigDefault->setToolTip("Restaurar configuraes\noriginais do sistema");
	
	getCheckTextoSecundario();

	connect( chkHabilitarTexto, SIGNAL( clicked() ), this, SLOT( chamarSetCheckTextoSecundario() ) );
	connect( btnTextoPrincipal, SIGNAL( clicked() ), this, SLOT( chamarSetTextoPrincipal() ) );
	connect( btnTextoSecundario, SIGNAL( clicked() ), this, SLOT( chamarSetTextoSecundario() ) );
	connect( btnConfigDefault, SIGNAL( clicked() ), this, SLOT( chamarCarregarConfiguracoesDefault() ) );
	
	boxTextos->setGeometry( QRect( 10, 10, 380, 310 ) );
	chkHabilitarTexto->setGeometry( QRect( 190, 10, 180, 30 ) );
	boxVersoes->setGeometry( QRect( 10, 35, 360, 80 ) );
	cmbVersoesTexto->setGeometry( QRect( 10, 30, 340, 25 ) );
	boxTextoPrincipal->setGeometry( QRect( 10, 125, 360, 80 ) );
	ledTextoPrincipal->setGeometry( QRect( 10, 30, 260, 25 ) );
	btnTextoPrincipal->setGeometry( QRect( 280, 30, 70, 25 ) );
	boxTextoSecundario->setGeometry( QRect( 10, 215, 360, 80 ) );
	ledTextoSecundario->setGeometry( QRect( 10, 30, 260, 25 ) );
	btnTextoSecundario->setGeometry( QRect( 280, 30, 70, 25 ) );
	boxGeral->setGeometry( QRect( 10, 325, 380, 80 ) );
	btnConfigDefault->setGeometry( QRect( 90, 30, 200, 25 ) );
	
	btnCancelar->setGeometry( QRect( width() - 80, height() - 35, 70, 25 ) );
}

void DialConfiguracoes::chamarSetCheckTextoSecundario(){
    AMessageBox aMsgBox;
    unsigned short int estado;
    bool flag = aMsgBox.yesNo("Aps aplicar esta configurao a " + (const QString&)NOME + " ser fechada e voc precisar abri-la novamente.\nDeseja continuar?",
                "Habilitar/desabilitar texto secundrio");
    if (flag){
        estado = 0;
	if (chkHabilitarTexto->isChecked())
            estado = 1;
	config.setCheckTextoSecundario(estado);
	
        aMsgBox.ok("A configurao foi executada com sucesso. Clique em OK para fechar a " + (const QString&)NOME,
        "Reiniciar " + (const QString&)NOME);
        this->parentWidget()->close();
    }
    else {
        if (chkHabilitarTexto->isChecked())
            chkHabilitarTexto->setChecked(false);
        else
            chkHabilitarTexto->setChecked(true);
    }

}

void DialConfiguracoes::getCheckTextoSecundario(){
	if (config.getCheckTextoSecundario()){
		chkHabilitarTexto->setChecked(true);
		ledTextoSecundario->setEnabled(true);
		btnTextoSecundario->setEnabled(true);
	}
	else{
		chkHabilitarTexto->setChecked(false);
		ledTextoSecundario->setEnabled(false);
		btnTextoSecundario->setEnabled(false);
	}
}

void DialConfiguracoes::chamarSetTextoPrincipal(){
	config.setTextoPrincipal(cmbVersoesTexto->currentText());
	ledTextoPrincipal->setText(config.getTextoPrincipal());
	btnCancelar->setText("Fechar");
	btnCancelar->setDefault(true);
}

void DialConfiguracoes::chamarSetTextoSecundario(){
	config.setTextoSecundario(cmbVersoesTexto->currentText());
	ledTextoSecundario->setText(config.getTextoSecundario());
	btnCancelar->setText("Fechar");
	btnCancelar->setDefault(true);
}

void DialConfiguracoes::chamarCarregarConfiguracoesDefault(){
	AMessageBox aMsgBox;
	bool flag = aMsgBox.yesNo("Esta opo ir excluir todas as suas configuraes atuais.\n"
	"Estudos e Anotaes no sero excludos.\nDeseja continuar?","Restaurar configuraes default");
	if (flag){
		Ferramentas ferramentas;
		ferramentas.rw(DIR_CONFIG_TEXTO, 'w', "");
		aMsgBox.ok(" necessrio reiniciar a " + (const QString&)NOME + " para que as novas configuraes funcionem corretamente!", "Reiniciar " + (const QString&)NOME);
	}
}

