/******************************************************************************
CLASSE READESTUDOS
CRIADO: 28-05-2008
ALTERADO: 29-01-2009
AUTOR: LUIS VASQUEZ - luis@comunidadeabiblia.net
WEB: www.comunidadeabiblia.net
--------------------------------aBblia Software-------------------------------
******************************************************************************/
#include "readestudos.h"
#include "readestudosvisualizar.h"
#include "amessagebox.h"
#include "bbcodehtml.h"
#include "diageditor.h"


/*Construdor com parmetros*/
ReadEstudos::ReadEstudos(const QString &categoriaLocal, const QString &tituloLocal, const QString &modoLocal):Read(){
	unsigned short int FONTE_MEDIA = resolucao.getFonteMedia();
	unsigned short int FONTE_PEQUENA = resolucao.getFontePequena();
	setModo(modoLocal);
	setCategoria(categoriaLocal);
	setTitulo(tituloLocal);
	setFormatar(false);
	setSalvo(true);

	QFont fonte = QFont("Bitstream", FONTE_MEDIA);

	// Tamanho
	setMinimumSize( QSize( 600, 500 ) ); resize(600, 500);

	//Detalhes
	ledTitulo = new QLineEdit(this);
	ledCategoria = new QLineEdit(this);
	ledAutor = new QLineEdit(this);
	dedData = new QDateEdit(this);
	dedData->setDisplayFormat("dd.MM.yyyy");
	tedRevisor = new QTextEdit(this);
	dedDataRevisao = new QDateEdit(this);
	dedDataRevisao->setDisplayFormat("dd.MM.yyyy");
	lblEstudo = new QLabel(this);
	lblTitulo = new QLabel(this);
	lblCategoria = new QLabel(this);
	lblAutor = new QLabel(this);
	lblData = new QLabel(this);
	lblRevisor = new QLabel(this);
	lblDataRevisao = new QLabel(this);
	fonte = QFont("Bitstream", config.getPointSizeTexto("ESTUDOS"));
	caixaTexto->setFont( fonte );
	fonte = QFont("Bitstream", FONTE_MEDIA);
	ledTitulo->setFont(fonte);
	ledCategoria->setFont(fonte);
	ledAutor->setFont(fonte);
	dedData->setFont(fonte);
	tedRevisor->setFont(fonte);
	dedDataRevisao->setFont(fonte);
	ledTitulo->setToolTip("Ttulo do estudo");
	ledCategoria->setToolTip("Categoria do estudo");
	ledAutor->setToolTip("Nome do autor");
	dedData->setToolTip("Data de criao");
	tedRevisor->setToolTip("Nomes dos revisores");
	dedDataRevisao->setToolTip("Data da ltima reviso");

	fonte = QFont("Bitstream", FONTE_PEQUENA);
	lblEstudo->setText("Estudo");
	lblEstudo->setFont(fonte);
	lblEstudo->setAlignment(Qt::AlignLeft);
	lblTitulo->setText("Ttulo");
	lblTitulo->setFont(fonte);
	lblCategoria->setText("Categoria");
	lblCategoria->setFont(fonte);
	lblAutor->setText("Autor");
	lblAutor->setFont(fonte);
	lblData->setText("Data de criao");
	lblData->setFont(fonte);
	lblRevisor->setText("Revisores");
	lblRevisor->setFont(fonte);
	lblDataRevisao->setText("Data da ltima reviso");
	lblDataRevisao->setFont(fonte);

	//Preenchendo campos
	BancoUser bancoUser;
	if (getModo() == "l"){
		BBCodeHtml bbToHtml;
		caixaTexto->setHtml(bbToHtml.convert(bancoUser.getConteudoEstudo(getCategoria(), getTitulo())));
	}
	if (getModo() == "e")
		caixaTexto->setText(bancoUser.getConteudoEstudo(getCategoria(), getTitulo()));

	if (getModo() != "c"){
		setWindowTitle(getTitulo() + " (" + getCategoria() + ")");
		ledTitulo->setText(getTitulo());
		ledCategoria->setText(getCategoria());
		ledAutor->setText(bancoUser.getAutorEstudo(getCategoria(), getTitulo()));
		v.setValue(bancoUser.getDataEstudo(getCategoria(), getTitulo()));
		dedData->setDate(v.toDate());
		tedRevisor->setText(bancoUser.getRevisorEstudo(getCategoria(), getTitulo()));
		v.setValue(bancoUser.getDataRevisaoEstudo(getCategoria(), getTitulo()));
		dedDataRevisao->setDate(v.toDate());
	}
	else if (getModo() == "c"){
		setWindowTitle("Estudo sem ttulo");
		dedData->setDate(QDate::currentDate());
		dedDataRevisao->setDate(QDate::currentDate());
	}

	//Marcando campos como editveis ou no
	if (getModo() == "l"){
		caixaTexto->setReadOnly(true);
		ledTitulo->setReadOnly(true);
		ledCategoria->setReadOnly(true);
		ledAutor->setReadOnly(true);
		dedData->setReadOnly(true);
		tedRevisor->setReadOnly(true);
		dedDataRevisao->setReadOnly(true);
	}
	else if (getModo() != "l"){
		caixaTexto->setReadOnly(false);
		ledTitulo->setReadOnly(false);
		ledCategoria->setReadOnly(false);
		ledAutor->setReadOnly(false);
		dedData->setReadOnly(false);
		tedRevisor->setReadOnly(false);
		dedDataRevisao->setReadOnly(false);
	}

	QObject::connect(caixaTexto,SIGNAL(textChanged() ),this,SLOT(slotModificado() ) );
	QObject::connect(ledCategoria,SIGNAL(textChanged(const QString &) ),this,
		SLOT(slotValidarModificacao(const QString &) ) );
	QObject::connect(ledTitulo,SIGNAL(textChanged(const QString &) ),this,
		SLOT(slotValidarModificacao(const QString &) ) );
	QObject::connect(ledAutor,SIGNAL(textChanged(const QString &) ),this,
		SLOT(slotValidarModificacao(const QString &) ) );
	QObject::connect(dedData,SIGNAL(textChanged(const QString &) ),this,
		SLOT(slotValidarModificacao(const QString &) ) );
	QObject::connect(tedRevisor,SIGNAL(textChanged(const QString &) ),this,
		SLOT(slotValidarModificacao(const QString &) ) );
	QObject::connect(dedDataRevisao,SIGNAL(textChanged(const QString &) ),this,
		SLOT(slotValidarModificacao(const QString &) ) );

	//Menus
	if (getModo() != "l"){
		caixaTexto->setAcceptRichText(false);

		salvar = new QAction(tr("&Salvar"), this);
		salvar->setShortcut(tr("Ctrl+S"));
		salvar->setStatusTip(tr("Salvar o estudo"));
		icon.load((const QString&)DIR_ICONS + "save.png");
		salvar->setIcon(icon);

		actRecarregar = new QAction(tr("&Recarregar"), this);
		actRecarregar->setShortcut(tr("Ctrl+R"));
		actRecarregar->setStatusTip(tr("Recarregar o estudo"));
		icon.load((const QString&)DIR_ICONS + "reload.png");
		actRecarregar->setIcon(icon);

		actVisualizar = new QAction(tr("Pr-visualizar"), this);
		actVisualizar->setShortcut(tr("F9"));
		actVisualizar->setStatusTip(tr("Visualizar o estudo"));
		icon.load((const QString&)DIR_ICONS + "preview.png");
		actVisualizar->setIcon(icon);
		
		actExibirFormatar = new QAction(tr("Formatar"), this);
		actExibirFormatar->setShortcut(tr("F10"));
		actExibirFormatar->setStatusTip(tr("Exibir o editor de formatao"));
		
		QObject::connect(salvar,SIGNAL(triggered() ),this,SLOT(slotSalvar() ) );
		QObject::connect(actRecarregar,SIGNAL(triggered() ),this,SLOT(slotRecarregar() ) );
		QObject::connect(this,SIGNAL(salvarTexto() ),this,SLOT(slotSalvar() ) );
		QObject::connect(actVisualizar,SIGNAL(triggered() ),this,SLOT(slotVisualizar() ) );
		QObject::connect(actExibirFormatar,SIGNAL(triggered() ),this,SLOT(slotFormatar() ) );

		arquivoMenu->addAction(salvar);
		arquivoMenu->addAction(actRecarregar);
		arquivoMenu->addAction(imprimir);
		arquivoMenu->addSeparator();
		arquivoMenu->addAction(sair);

		exibirMenu->addAction(actExibirFormatar);
		exibirMenu->addSeparator();
		exibirMenu->addAction(aumentarFonte);
		exibirMenu->addAction(diminuirFonte);
		exibirMenu->addAction(actFontePadrao);
		exibirMenu->addSeparator();
		exibirMenu->addAction(actVisualizar);
		exibirMenu->addAction(exibirTelaInteira);
	}

	QObject::connect(aumentarFonte,SIGNAL(triggered() ),this, SLOT(slotZoomIn() ) );
	QObject::connect(diminuirFonte,SIGNAL(triggered() ),this, SLOT(slotZoomOut() ) );
	QObject::connect(actFontePadrao,SIGNAL(triggered() ),this, SLOT(slotZoomDefault() ) );
}
//Menu
bool ReadEstudos::slotSalvar(){
	BancoUser bancoUser;
	AMessageBox aMsgBox;
	if (ledTitulo->text().trimmed().isEmpty()){
		aMsgBox.ok("O campo 'Ttulo' est vazio!", "Campo vazio");
	}
	else if (ledCategoria->text().trimmed().isEmpty()){
		aMsgBox.ok("O campo 'Categoria' est vazio!", "Campo vazio");
	}
	else {
		QString data, dataRevisao;
		v.setValue(dedData->date());
		data = v.toString();
		v.setValue(dedDataRevisao->date());
		dataRevisao = v.toString();
		if (getModo() == "c"){
			if (bancoUser.existEstudo(ledCategoria->text().trimmed(), ledTitulo->text().trimmed())){
				aMsgBox.ok("O Ttulo informado j existe para essa Categoria!", "Ttulo j existe");
			}
			else {
				bancoUser.setEstudo(ledCategoria->text().trimmed(), ledTitulo->text().trimmed(),
				caixaTexto->toPlainText().trimmed(), ledAutor->text().trimmed(),
				data,tedRevisor->toPlainText().trimmed(),dataRevisao);
				setModo("e");
				setSalvo(true);
			 }
		}
		else if (getModo() == "e") {
				if ( (getTitulo() != ledTitulo->text().trimmed()) || (getCategoria() != ledCategoria->text().trimmed()) ){
					setSalvo(false);
					if (bancoUser.existEstudo(ledCategoria->text().trimmed(), ledTitulo->text().trimmed()))
						aMsgBox.ok("O Ttulo informado j existe para essa Categoria!", "Ttulo j existe");
					else
						setSalvo(true);
				}
				else
						setSalvo(true);
				if (getSalvo()){
					bancoUser.editEstudo(getCategoria(), getTitulo(), ledCategoria->text().trimmed(),
					ledTitulo->text().trimmed(), caixaTexto->toPlainText().trimmed(),
					ledAutor->text().trimmed(), data,
					tedRevisor->toPlainText().trimmed(),dataRevisao);
				}
		}
		if (getSalvo()){
			setWindowTitle(ledTitulo->text().trimmed() + " (" + ledCategoria->text().trimmed() + ")");
                        setCategoria(ledCategoria->text().trimmed());
                        setTitulo(ledTitulo->text().trimmed());
			emit tituloAlterado(windowTitle());
			emit update();
			return true;
		}
	}
	return false;
}
void ReadEstudos::slotRecarregar(){
	BancoUser bancoUser;
	
	caixaTexto->setText(bancoUser.getConteudoEstudo(getCategoria(),getTitulo()));
	ledTitulo->setText(getTitulo());
	ledCategoria->setText(getCategoria());
	ledAutor->setText(bancoUser.getAutorEstudo(getCategoria(),getTitulo()));
	v.setValue(bancoUser.getDataEstudo(getCategoria(),getTitulo()));
	dedData->setDate(v.toDate());
	tedRevisor->setText(bancoUser.getRevisorEstudo(getCategoria(),getTitulo()));
	v.setValue(bancoUser.getDataRevisaoEstudo(getCategoria(),getTitulo()));
	dedDataRevisao->setDate(v.toDate());
	setSalvo(true);
	setWindowTitle(getTitulo() + " (" + getCategoria() + ")");
        setCategoria(ledCategoria->text().trimmed());
        setTitulo(ledTitulo->text().trimmed());
}
void ReadEstudos::slotZoomIn(){
	caixaTexto->zoomIn();
	QFont fonteAtual = caixaTexto->currentFont();
	short int pointSize = fonteAtual.pointSize();
	config.setPointSizeTexto(pointSize, "ESTUDOS");
}
void ReadEstudos::slotZoomOut(){
	caixaTexto->zoomOut();
	QFont fonteAtual = caixaTexto->currentFont();
	short int pointSize = fonteAtual.pointSize();
	config.setPointSizeTexto(pointSize, "ESTUDOS");
}
void ReadEstudos::slotZoomDefault(){
	config.setPointSizeTexto(resolucao.getFonteMedia(), "ESTUDOS");
	QFont fonte = QFont("Bitstream", config.getPointSizeTexto("ESTUDOS"));
	caixaTexto->setFont( fonte );
}
void ReadEstudos::slotModificado(){
	if (getModo() == "e"){
		if (ledTitulo->text().trimmed().isEmpty() || getTitulo().trimmed().isEmpty())
			setWindowTitle("* Estudo sem ttulo");
		else
			setWindowTitle("* " + getTitulo() + " (" + getCategoria() + ")");
	}
	else if (getModo() == "c")
		setWindowTitle("* Estudo sem ttulo");
	setSalvo(false);
	emit tituloAlterado(windowTitle());
}
void ReadEstudos::slotValidarModificacao(){
	bool confirm = false;
	BancoUser bancoUser;
	
	if (getModo() == "e"){
		if (ledCategoria->text().trimmed() != getCategoria() || ledTitulo->text().trimmed() != getTitulo() || ledAutor->text().trimmed() != bancoUser.getAutorEstudo(getCategoria(),getTitulo()))
			confirm = true;
		v.setValue(dedData->date());
		if (v.toString() != bancoUser.getDataEstudo(getCategoria(),getTitulo()))
			confirm = true;
		v.setValue(dedDataRevisao->date());
		if (v.toString() != bancoUser.getDataRevisaoEstudo(getCategoria(),getTitulo()))
			confirm = true;
	}
	else if (getModo() == "c"){
		if (ledCategoria->text().trimmed() != "" || ledTitulo->text().trimmed() != "" || ledAutor->text().trimmed() != "")
			confirm = true;
		if (dedData->date() != QDate::currentDate() || dedDataRevisao->date() != QDate::currentDate())
			confirm = true;
	}
	if (confirm)
		slotModificado();
}
void ReadEstudos::slotValidarModificacao(const QString &){
	slotValidarModificacao();
}
void ReadEstudos::slotVisualizar(){
	QString data, dataRevisao;
	v.setValue(dedData->date());
	data = v.toString();
	v.setValue(dedDataRevisao->date());
	dataRevisao = v.toString();
	ReadEstudosVisualizar* wgtVisualizar = new ReadEstudosVisualizar(ledCategoria->text().trimmed(),
					ledTitulo->text().trimmed(), caixaTexto->toPlainText().trimmed(),
					ledAutor->text().trimmed(), data,
					tedRevisor->toPlainText().trimmed(),dataRevisao);
	wgtVisualizar->setParent(this->parentWidget(), Qt::Window);
	wgtVisualizar->show();
}
void ReadEstudos::slotFormatar(){
	if (!getFormatar()){
		mdiSubWindow = new DiagEditor( this, Qt::Dialog );
		mdiSubWindow->show();
		setFormatar(true);

		connect(mdiSubWindow,SIGNAL(clickedFormato(const QString&) ),this,SLOT(slotAddFormato(const QString&) ) );
		connect(mdiSubWindow,SIGNAL(finalizado() ),this,SLOT(slotFecharFormatar() ) );
	}
	else{
		mdiSubWindow->hide();
		setFormatar(false);
		delete mdiSubWindow;
	}
}
void ReadEstudos::slotFecharFormatar(){
	setFormatar(false);
}
void ReadEstudos::slotAddFormato(const QString &valor){
	QString selecao = caixaTexto->textCursor().selectedText();
	if (valor != "clear"){
		QString strIni = valor.section( '|', 0, 0);
		QString strFim = valor.section( '|', 1, 1);

		if (selecao.isEmpty())
			caixaTexto->insertPlainText(strIni + strFim);
		else
			caixaTexto->insertPlainText(strIni + selecao + strFim);
	}
	else {
		if (selecao.isEmpty()){
			caixaTexto->selectAll();
			selecao = caixaTexto->textCursor().selectedText();
		}
		BBCodeHtml list;
		int posI, posF;
		bool start, stop, flag = true;
		while (flag){
			posI = posF = 0;
			start = stop = false;
			for (int i = 0; i < selecao.size(); i++){
				if (i == selecao.size() - 1)
					flag = false;
				if (selecao[i] == '['){
					posI = i;
					start = true;
				}
				if ((selecao[i] == ']') && (start)){
					posF = i;
					for (int j = 0; j < list.listBBCode.size(); j++)
						if ((posI == 0) && (selecao.left(posF + 1).contains(list.listBBCode.at(j), Qt::CaseInsensitive))){
							selecao.remove(0, posF + 1);
							stop = true;
							break;
						}
						else if (selecao.mid(posI, posF).contains(list.listBBCode.at(j), Qt::CaseInsensitive)){
							selecao.remove(posI, posF - posI + 1);
							stop = true;
							break;
						}
					if (stop)
						break;
				}
			}
		}
		caixaTexto->insertPlainText(selecao);
	}
}

//Recupera variveis
QString ReadEstudos::getCategoria(){
	return _strCategoria;
}
void ReadEstudos::setCategoria(const QString &strCategoria){
	_strCategoria = strCategoria;
}
QString ReadEstudos::getTitulo(){
	return _strTitulo;
}
void ReadEstudos::setTitulo(const QString &strTitulo){
	_strTitulo = strTitulo;
}
QString ReadEstudos::getModo(){
	return _strModo;
}
void ReadEstudos::setModo(const QString &strModo){
	_strModo = strModo;
}
bool ReadEstudos::getFormatar(){
	return _boolFormatar;
}
void ReadEstudos::setFormatar(bool boolFormatar){
	_boolFormatar = boolFormatar;
}

//Redimencionador
void ReadEstudos::resizeEvent(QResizeEvent * /*event*/){
	int extraWidth = width() - minimumWidth();
	int extraHeight = height() - minimumHeight();

	comunidade->setGeometry(10, 480 + extraHeight, 250, 21 + 4);
	caixaTexto->setGeometry(10, 50, 580 + extraWidth, 300 + extraHeight);
	ledTitulo->setGeometry(10, 370 + extraHeight, 370 + extraWidth/2, 21 + 4);
	ledCategoria->setGeometry(390 + extraWidth/2, 370 + extraHeight, 200 + extraWidth/2, 21 + 4);
	ledAutor->setGeometry(10, 420 + extraHeight, 220 + extraWidth/2, 21 + 4);
	dedData->setGeometry(240 + extraWidth/2, 420 + extraHeight, 100, 21 + 4);
	tedRevisor->setGeometry(350 + extraWidth/2, 420 + extraHeight, 240 + extraWidth/2, 65);
	dedDataRevisao->setGeometry(240 + extraWidth/2, 460 + extraHeight, 100, 21 + 4);
	//Label
	lblEstudo->setGeometry(10, 36, 580 + extraWidth, 15);
	lblTitulo->setGeometry(10, 354 + extraHeight, 370, 20);
	lblCategoria->setGeometry(390 + extraWidth/2, 354 + extraHeight, 370, 20);
	lblAutor->setGeometry(10, 404 + extraHeight, 370, 20);
	lblData->setGeometry(240 + extraWidth/2, 404 + extraHeight, 370, 20);
	lblRevisor->setGeometry(350 + extraWidth/2, 404 + extraHeight, 370, 20);
	#if defined(__linux__)
		lblDataRevisao->setGeometry(105 + extraWidth/2, 465 + extraHeight, 130, 20);
	#elif defined(_WIN32) || defined(__WIN32) || defined(WIN32) || defined(__WIN32__)
		lblDataRevisao->setGeometry(125 + extraWidth/2, 465 + extraHeight, 110, 20);
	#endif
}

//Fechar Janela
void ReadEstudos::closeEvent(QCloseEvent *event){
	int flag = 0;
	if (!getSalvo()){
		AMessageBox aMsgBox;
		flag = aMsgBox.yesNoCancel("O texto no foi salvo.\nDeseja salv-lo agora?", "Sair");
		if (flag == 0){
			liberarMemoria();
			event->ignore(); emit finalizado(); //Ignore, ser finalizado pela aplicaao Pai
		}
		else if (flag == 1){
			if (slotSalvar()){
				liberarMemoria();
				event->ignore(); emit finalizado();
			}
			else
				event->ignore();
		}
		else
			event->ignore();
	}
	else {
		liberarMemoria();
		event->ignore(); emit finalizado();
	}
}

void ReadEstudos::liberarMemoria(){
	delete comunidade;
	delete lblEstudo;
	delete caixaTexto;
	delete ledTitulo;
	delete ledCategoria;
	delete ledAutor;
	delete dedData;
	delete tedRevisor;
	delete dedDataRevisao;
	delete lblTitulo;
	delete lblCategoria;
	delete lblAutor;
	delete lblData;
	delete lblRevisor;
	delete lblDataRevisao;
}

ReadEstudos::~ReadEstudos(){
	close();
}

/*Construdor sem parmetros herdado pela classe filha ReadEstudosVisualizar*/
ReadEstudos::ReadEstudos(){
	int FONTE_MEDIA = resolucao.getFonteMedia();
	int FONTE_PEQUENA = resolucao.getFontePequena();
	setSalvo(true);

	QFont fonte = QFont("Bitstream", FONTE_MEDIA);

	// Tamanho
	setMinimumSize( QSize( 600, 500 ) ); resize(600, 500);

	//Detalhes
	ledTitulo = new QLineEdit(this);
	ledCategoria = new QLineEdit(this);
	ledAutor = new QLineEdit(this);
	dedData = new QDateEdit(this);
	dedData->setDisplayFormat("dd.MM.yyyy");
	tedRevisor = new QTextEdit(this);
	dedDataRevisao = new QDateEdit(this);
	dedDataRevisao->setDisplayFormat("dd.MM.yyyy");
	lblEstudo = new QLabel(this);
	lblTitulo = new QLabel(this);
	lblCategoria = new QLabel(this);
	lblAutor = new QLabel(this);
	lblData = new QLabel(this);
	lblRevisor = new QLabel(this);
	lblDataRevisao = new QLabel(this);
	ledTitulo->setFont(fonte);
	ledCategoria->setFont(fonte);
	ledAutor->setFont(fonte);
	dedData->setFont(fonte);
	tedRevisor->setFont(fonte);
	dedDataRevisao->setFont(fonte);

	fonte = QFont("Bitstream", FONTE_PEQUENA);
	lblEstudo->setText("Estudo");
	lblEstudo->setFont(fonte);
	lblEstudo->setAlignment(Qt::AlignLeft);
	lblTitulo->setText("Ttulo");
	lblTitulo->setFont(fonte);
	lblCategoria->setText("Categoria");
	lblCategoria->setFont(fonte);
	lblAutor->setText("Autor");
	lblAutor->setFont(fonte);
	lblData->setText("Data de criao");
	lblData->setFont(fonte);
	lblRevisor->setText("Revisores");
	lblRevisor->setFont(fonte);
	lblDataRevisao->setText("Data da ltima reviso");
	lblDataRevisao->setFont(fonte);
	
	QObject::connect(aumentarFonte,SIGNAL(triggered() ),this, SLOT(slotZoomIn() ) );
	QObject::connect(diminuirFonte,SIGNAL(triggered() ),this, SLOT(slotZoomOut() ) );
	QObject::connect(actFontePadrao,SIGNAL(triggered() ),this, SLOT(slotZoomDefault() ) );
}

